<?php

class Lieferland
{
	private $oPlugin;
	private $countries;
	private $countries_assoc;
	
	public function __construct(Plugin $oPlugin)
	{
		$this->oPlugin = $oPlugin;
		$this->Sprachvariablen = new PluginSprachVariablen($this->oPlugin->oPluginSprachvariable_arr);
		
		
		if((int)$this->oPlugin->nCalledHook == HOOK_INDEX_NAVI_HEAD_POSTGET)
		{
			//Set global Variables in Smarty for Plugintemplates and Boxes
			$this->countries = Shop::DB()->query("SELECT DISTINCT l.* FROM tland AS l JOIN tversandart AS v ON v.cLaender LIKE CONCAT('%', l.cISO, '%') GROUP BY l.cISO ORDER BY l.cDeutsch ASC", 2);
			foreach($this->countries as $c) {
				$this->countries_assoc[$c->cISO] = $c;
			}
			Shop::Smarty()->assign('km_lieferlaender',$this->countries);
			Shop::Smarty()->assign('km_lieferlaender_assoc', $this->countries_assoc);
			Shop::Smarty()->assign('km_lieferlaender_sprache',$this->oPlugin->oPluginSprachvariable_arr);
			
			//Sprachvariablen
			$lang = strtoupper($this->getLanguage());
			Shop::Smarty()->assign("lieferland_label",          	$this->Sprachvariablen->getText('xmlp_lieferland_label', $lang));
			Shop::Smarty()->assign('km_header_setlocation',         $this->Sprachvariablen->getText('xmlp_header_setlocation',  $lang));
			Shop::Smarty()->assign('km_country_setlocation',        $this->Sprachvariablen->getText('xmlp_country_setlocation', $lang));
			Shop::Smarty()->assign('km_btn_setlocation',            $this->Sprachvariablen->getText('xmlp_button_setlocation',  $lang));
			Shop::Smarty()->assign('tlieferland_lang',              strtolower($lang));
		}
	}
	
	public function autoDetect()
	{
		$ll = false;
		
		//Via Cloudflare erkennen?
		if($this->getValue('km_lieferland_type') != 3 && !empty($_SERVER["HTTP_CF_IPCOUNTRY"]))	//Wenn 3 dann nur direkt via IP!
			$ll = $_SERVER["HTTP_CF_IPCOUNTRY"];
			
		if($this->isValidCode($ll)) return $ll;
		
		//Wenn Lieferland noch unbekannt nun via IP erkennen
		$country = json_decode(file_get_contents('https://ipinfo.io/'.$_SERVER['REMOTE_ADDR']));
		$ll = isset($country->country) ? $country->country : null;
		
		return $ll;
	}
	
	public function isValidCode($iso)
	{
        if(empty($iso)) {
            return false;
        }
		
        return (($this->isISOCompliant($iso) == true) && ($this->isAlphaCompliant($iso) == true));
	}
	
	public function useAutoDetection()
	{
		return ($this->getValue('km_lieferland_type') >= 3);
	}
	
	public function showDialog()
	{
		//Kein Lieferland bisher gesetzt, daher den Dialog dazu automatisch öffnen
		pq('body')->append('<script type="text/javascript" defer src="data:text/javascript;base64,'.base64_encode(Shop::Smarty()->fetch($this->oPlugin->cFrontendPfad.'js/showDialog.js')).'"></script>');
	}
	
	public function dialogPossible()
	{
		//Ist das Standardland aktiv?
		if($this->useAutoDetection())
			return false; //Bei automatischer Erkennung kein Dialog anzeigen!
		if(trim($this->getValue('km_lieferland_auto')) != "" 
			&& in_array($this->autoDetect(),explode(';',$this->getValue('km_lieferland_auto')))
			&& !isset($_SESSION['ll_set'])
		)
			return false;
			
		//Ansonsten ist der Dialog möglich (auch für Autoerkennung im Notfall anzeigen)
		return true;
	}
	
	public function integrateTemplates()
	{
		//Template definieren und einbinden, auch für Boxenverwaltung
		$template = ($this->getValue('km_lieferland_type') == 2) ? 'flags' : 'dropdown';
		pq('body')->append(
			Shop::Smarty()->fetch($this->oPlugin->cFrontendPfad.'template/'.$template.'.tpl').
			'<script type="text/javascript" defer src="data:text/javascript;base64,'.base64_encode(Shop::Smarty()->fetch($this->oPlugin->cFrontendPfad.'js/box.js')).'"></script>'
		);
		pq('head')->append('<style>'.Shop::Smarty()->fetch($this->oPlugin->cFrontendPfad.'css/main.css').'</style>');
	}

    private function getValue($key) : string {
        return (isset($this->oPlugin->oPluginEinstellungAssoc_arr[$key]) ? $this->oPlugin->oPluginEinstellungAssoc_arr[$key] : "");
    }

    private function isAlphaCompliant(string $str) : bool {
        return ctype_alpha($str);
    }

    private function isISOCompliant(string $iso) : bool  {
        return ((strlen($iso) == 2) || (strlen($iso) == 3));
    }
	
    private function getLanguage() : string {
        return strtoupper(Shop::DB()->query("select cISO from tsprache where kSprache=".$this->getValue('km_language').";", 1)->cISO);
    }
}

class PluginSprachVariablen {
    private $Sprachvariablen;

    public function __construct($arr) {
        $this->Sprachvariablen = $arr;
    }

    public function getObject(string $cName) {
        foreach($this->Sprachvariablen as $v) {
            if($v->cName == $cName) {
                return $v;
            }
        }
        return null;
    }

    public function getText(string $cName, string $iso) : string {
        if(empty($iso)|| empty($cName)) {
            return '';
        }
        foreach($this->Sprachvariablen as $v) {
            if (strtolower($v->cName) == strtolower($cName) && array_key_exists(strtoupper($iso), $v->oPluginSprachvariableSprache_arr)) {
                return $v->oPluginSprachvariableSprache_arr[$iso];
            }
        }
        return '';
    }

}
?>